@@ -37,21 +37,19 @@ typicalize <- function(file.sim,lines.sim,file.mod,return.text=FALSE,file.ext,Ne
 
     ## OMEGAP and OMEGAPD for NWPRI
     if("OMEGAP"%in%names(sections.sim)) {
-        if(packageVersion("NMdata")<"0.1.6.932"){
-            message("NMdata version 0.1.7 or newer needed for typical=TRUE to handle OMEGAP sections. Not handling $OMEGAP.")
-        } else {
-            lines.omegap <- paste(c("$OMEGAP",rep("1E-30 FIX",Netas),""),collapse="\n")
-            lines.sim <- NMdata:::NMwriteSectionOne(lines=lines.sim,section="omegap",newlines=lines.omegap,backup=FALSE,quiet=TRUE)
-        }
+
+            ## lines.omegap <- paste(c("$OMEGAP",rep("1E-30 FIX",Netas),""),collapse="\n")
+            ## lines.sim <- NMdata:::NMwriteSectionOne(lines=lines.sim,section="omegap",newlines=lines.omegap,backup=FALSE,quiet=TRUE)
+            lines.sim <-
+                gsub ("\\$OMEGAP *(FIX|FIXED)* *[0-9.]+ *(FIX|FIXED)* *","$OMEGAP 0 FIX",lines.sim)
+
     }
 
     if("OMEGAPD"%in%names(sections.sim)) {
-        if(packageVersion("NMdata")<"0.1.6.932"){
-            message("NMdata version 0.1.7 or newer needed for typical=TRUE to handle OMEGAPD sections. Not handling $OMEGAPD.")
-        } else {
-            lines.omegapd <- paste(c("$OMEGAPD",rep("1 FIX",Netas),""),collapse="\n")
-            lines.sim <- NMdata:::NMwriteSectionOne(lines=lines.sim,section="omegapd",newlines=lines.omegapd,backup=FALSE,quiet=TRUE)
-        }
+            ## lines.omegapd <- paste(c("$OMEGAPD",rep("1 FIX",Netas),""),collapse="\n")
+            ## lines.sim <- NMdata:::NMwriteSectionOne(lines=lines.sim,section="omegapd",newlines=lines.omegapd,backup=FALSE,quiet=TRUE)
+        lines.sim <- gsub ("\\$OMEGAPD *(FIX|FIXED)* *[0-9.]+ *(FIX|FIXED)* *","$OMEGAPD 0 FIX",lines.sim)
+
     }
     
     if(return.text){
